package nl.tudelft.ewi.es.mbdlistener.diagnoser;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.Set;
import java.util.TreeSet;

public class Diagnoser {
	Set<Conflict> conflictSet = new TreeSet<Conflict>();
	private ArrayList<Diagnosis> diagnoses = new ArrayList<Diagnosis>();
	private ArrayList<Diagnosis> sensorDiagnoses = new ArrayList<Diagnosis>();
	private ArrayList<Diagnosis> nodeDiagnoses = new ArrayList<Diagnosis>();
	
	/**
	 * Add a conflict to the diagnoser.
	 * @param conflict
	 */
	public void addConflict(Conflict conflict) {
		if (conflictSet.contains(conflict)) return;
		conflictSet.add(conflict);
		
		if (conflict.getConflict().first() instanceof SensorEnvironment) { 
			sensorDiagnoses = addConflict(conflict, sensorDiagnoses);
		} else {
			nodeDiagnoses = addConflict(conflict, nodeDiagnoses);
		}		
	}


	public Conflict removeConflict(int id, int source) {
		for (Conflict conflict : conflictSet) {
			if (conflict.getId() == id && conflict.getSource() == source) {
				removeConflict(conflict);
				return conflict;
			}
		}
		return null;
	}
	
	/**
	 * Remove a conflict from the diagnoser. Results in a complete rebuild of the diagnoses.
	 * @param conflict
	 */
	public void removeConflict(Conflict conflict) {
		conflictSet.remove(conflict);
		
		if (conflict.getConflict().first() instanceof SensorEnvironment) {
			sensorDiagnoses.clear();
			for (Conflict oldConflict: conflictSet) {
				if (oldConflict.getConflict().first() instanceof SensorEnvironment) {
					sensorDiagnoses = addConflict(oldConflict, sensorDiagnoses);
				}
			}
		} else {
			nodeDiagnoses.clear();
			for (Conflict oldConflict: conflictSet) {
				if (!(oldConflict.getConflict().first() instanceof SensorEnvironment)) {
					nodeDiagnoses = addConflict(oldConflict, nodeDiagnoses);
				}
			}
		}
	}
	
	/**
	 * Add conflict to the specified diagnosis set
	 * @param conflict
	 * @param curDiagnoses
	 * @return
	 */
	private ArrayList<Diagnosis> addConflict(Conflict conflict, ArrayList<Diagnosis> curDiagnoses) {
		// Case empty, just add every entry of the conflict as a diagnosis
		if (curDiagnoses.isEmpty()) {
			for (Environment env : conflict.getConflict()) {
				curDiagnoses.add(new Diagnosis(env));
			}
			return curDiagnoses;
		}

		ArrayList<Diagnosis> newdiagnoses = new ArrayList<Diagnosis>();
		// A diagnosis already exists, add each conflict entry to every existing diagnosis, filtering a few
		for (Environment env : conflict.getConflict()) {
			for (Diagnosis diagnosis: curDiagnoses) {
				// Copy existing diagnosis
				Diagnosis newDiagnosis = new Diagnosis(diagnosis.getDiagnosis());
				// Add conflict entry
				newDiagnosis.add(env);

				// Filter non-minimal diagnoses
				ArrayList<Diagnosis> removeable = new ArrayList<Diagnosis>();
				boolean dontadd = false;
				for(Diagnosis existingDiagnosis : newdiagnoses) {
					// Skip the new diagnosis, if it is a superset of an existing diagnosis  
					if (newDiagnosis.supersetOf(existingDiagnosis)) {
						dontadd = true;
						break;
					}
					// Remove existing diagnosis, if it is a superset of our new diagnosis
					if (existingDiagnosis.supersetOf(newDiagnosis)){
						removeable.add(existingDiagnosis);
					}
				}
				// Actual remove
				newdiagnoses.removeAll(removeable);

				// Actual add
				if (!dontadd) {
					newdiagnoses.add(newDiagnosis);
				}
			}
		}
		
		return newdiagnoses;
	}

	/**
	 * Returns a list of the current diagnoses, ordered by their likelihood
	 * @return
	 */
	public List<Diagnosis> getDiagnoses() {
		diagnoses.clear();
		if (sensorDiagnoses.isEmpty())
			return nodeDiagnoses;
		if (nodeDiagnoses.isEmpty())
			return sensorDiagnoses;
		for (Diagnosis diagnosis : sensorDiagnoses) {
			for (Diagnosis diagnosis2 : nodeDiagnoses) {
				Diagnosis finalDiag = new Diagnosis(diagnosis.getDiagnosis());
				finalDiag.getDiagnosis().addAll(diagnosis2.getDiagnosis());
				diagnoses.add(finalDiag);
			}
		}
		
		Collections.sort(diagnoses);
		return diagnoses;
	}
	public Diagnosis getBestDiagnosis() {
		Collections.sort(sensorDiagnoses);
		Collections.sort(nodeDiagnoses);
		
		Diagnosis diagnosis = new Diagnosis();
		if (!sensorDiagnoses.isEmpty()) {
			for (Environment env : sensorDiagnoses.get(0).getDiagnosis()) {
				diagnosis.add(env);
			}
		}
		if (!nodeDiagnoses.isEmpty()) {
			for (Environment env : nodeDiagnoses.get(0).getDiagnosis()) {
				diagnosis.add(env);
			}
		}

		return diagnosis;
		
//		if (diagnoses.isEmpty())
//			return new Diagnosis();
//		return diagnoses.get(0);
	}

}
