package nl.tudelft.ewi.es.mbdlistener.graphics;

import java.util.ArrayList;
import java.util.List;
import java.util.Vector;

import javax.swing.event.TreeModelEvent;
import javax.swing.event.TreeModelListener;
import javax.swing.tree.TreeModel;
import javax.swing.tree.TreePath;

import nl.tudelft.ewi.es.mbdlistener.diagnoser.Diagnosis;
import nl.tudelft.ewi.es.mbdlistener.events.ConflictCancellationEvent;
import nl.tudelft.ewi.es.mbdlistener.events.ConflictEvent;

public class DiagnoserTreeModel implements TreeModel {
    private String rootNode = "Conflicts";
    private Vector<TreeModelListener> treeModelListeners = new Vector<TreeModelListener>();

	private List<DiagnoserEvent> events = new ArrayList<DiagnoserEvent>();

	@Override
	public void addTreeModelListener(TreeModelListener l) {
		treeModelListeners.addElement(l);
	}

	@Override
	public Object getChild(Object parent, int index) {
		if (parent == rootNode) {
			return events.get(index);
		}
		if (parent instanceof DiagnoserEvent) {
			DiagnoserEvent event = (DiagnoserEvent) parent;
			return event.getDiagnoses().get(index);
		}
		return null;
	}

	@Override
	public int getChildCount(Object parent) {
		if (parent == rootNode) {
			return events.size();
		}
		if (parent instanceof DiagnoserEvent) {
			DiagnoserEvent event = (DiagnoserEvent) parent;
			return event.getDiagnoses().size(); 
		}
		return 0;
	}

	@Override
	public int getIndexOfChild(Object parent, Object child) {
		if (parent == rootNode) {
			return events.indexOf(child);
		}
		if (parent instanceof DiagnoserEvent) {
			DiagnoserEvent event = (DiagnoserEvent) parent;
			return event.getDiagnoses().indexOf(child); 
		}
		return -1;
	}

	@Override
	public Object getRoot() {
		return rootNode;
	}

	@Override
	public boolean isLeaf(Object node) {
		if (node instanceof Diagnosis)
			return true;
		return false;
	}

	@Override
	public void removeTreeModelListener(TreeModelListener l) {
		treeModelListeners.removeElement(l);
	}

	@Override
	public void valueForPathChanged(TreePath path, Object newValue) {
        System.out.println("*** valueForPathChanged : " + path + " --> " + newValue);
	}

	public void addConflict(ConflictEvent conflictEvent, List<Diagnosis> diagnoses) {
		ConflictDiagnoserEvent event;
		event = new ConflictDiagnoserEvent(diagnoses, conflictEvent.getConflict());
		events.add(event);
		
		TreeModelEvent e = new TreeModelEvent(this, new Object[] {rootNode}, new int[] {events.size()-1}, new Object[] {event});
		for (TreeModelListener tml : treeModelListeners) {
			tml.treeNodesInserted(e);
		}
	}

	public void addCancelConflict(ConflictCancellationEvent conflictCancelEvent, List<Diagnosis> diagnoses) {
		CancelConflictDiagnoserEvent event;
		event = new CancelConflictDiagnoserEvent(diagnoses, conflictCancelEvent.getSource(), conflictCancelEvent.getId());
		events.add(event);

		TreeModelEvent e = new TreeModelEvent(this, new Object[] {rootNode}, new int[] {events.size()-1}, new Object[] {event});
		for (TreeModelListener tml : treeModelListeners) {
			tml.treeNodesInserted(e);
		}
	}
}
