package nl.tudelft.ewi.es.mbdlistener.graphics;

import java.awt.Dimension;
import java.awt.GridLayout;

import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSplitPane;
import javax.swing.JTree;
import javax.swing.UIManager;
import javax.swing.event.TreeSelectionEvent;
import javax.swing.event.TreeSelectionListener;
import javax.swing.tree.TreeSelectionModel;

import nl.tudelft.ewi.es.mbdlistener.diagnoser.Diagnosis;

public class MainPanel extends JPanel implements TreeSelectionListener {
	private static final long serialVersionUID = 1L;
	private static MainPanel printer;

	private DiagnoserTreeModel treeModel;
	private JTree tree;
	private GraphPane graphPane;
	
	public MainPanel() {
		super(new GridLayout(1,0));

		treeModel = new DiagnoserTreeModel();
		tree = new JTree(treeModel);
		tree.getSelectionModel().setSelectionMode(TreeSelectionModel.SINGLE_TREE_SELECTION);
		tree.addTreeSelectionListener(this);
		JScrollPane treePane = new JScrollPane(tree);
		treePane.setPreferredSize(new Dimension(300,800));
		
		graphPane = new GraphPane();
        JScrollPane textView = new JScrollPane(graphPane);
		textView.setPreferredSize(new Dimension(600,800));
		
		JSplitPane splitPane = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT);
		splitPane.setLeftComponent(treePane);
		splitPane.setRightComponent(textView);
		add(splitPane);
	}
	
	public static void runGui() {
		try {
			UIManager.setLookAndFeel(
					UIManager.getSystemLookAndFeelClassName());
		} catch (Exception e) {
			System.err.println("Couldn't use system look and feel.");
		}

		JFrame frame = new JFrame("Diagnosis results");
		frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		
		printer = new MainPanel();

		frame.add(printer);
		
		frame.pack();
		frame.setVisible(true);
	}


	public static DiagnoserTreeModel getTreeModel() {
		return printer.treeModel;
	}


	@Override
	public void valueChanged(TreeSelectionEvent e) {
	    Object node = tree.getLastSelectedPathComponent();
	    if (node == null)
	    	return;
	    
	    if (node instanceof Diagnosis) {
	    	graphPane.setDiagnosis((Diagnosis) node);
	    } 
	}
}
