package nl.tudelft.ewi.es.mbdlistener.output;

import java.io.OutputStream;
import java.util.List;
import java.util.Observable;

import javax.xml.stream.XMLOutputFactory;
import javax.xml.stream.XMLStreamException;
import javax.xml.stream.XMLStreamWriter;

import nl.tudelft.ewi.es.mbdlistener.diagnoser.Diagnoser;
import nl.tudelft.ewi.es.mbdlistener.diagnoser.Diagnosis;
import nl.tudelft.ewi.es.mbdlistener.diagnoser.Environment;
import nl.tudelft.ewi.es.mbdlistener.diagnoser.LinkEnvironment;
import nl.tudelft.ewi.es.mbdlistener.diagnoser.NodeEnvironment;
import nl.tudelft.ewi.es.mbdlistener.diagnoser.SensorEnvironment;
import nl.tudelft.ewi.es.mbdlistener.events.ConflictCancellationEvent;
import nl.tudelft.ewi.es.mbdlistener.events.ConflictEvent;
import nl.tudelft.ewi.es.mbdlistener.events.EndEvent;
import nl.tudelft.ewi.es.mbdlistener.events.MessageEvent;
import nl.tudelft.ewi.es.mbdlistener.events.NodeDeathEvent;
import nl.tudelft.ewi.es.mbdlistener.events.SensorDeathEvent;
import nl.tudelft.ewi.es.mbdlistener.events.StartEvent;

public class XmlStreamPrinter extends ListenerOutput {
	private XMLStreamWriter xtw;
	private final boolean writeDiagnoses;

	public XmlStreamPrinter(Diagnoser diagnoser, OutputStream stream, boolean writeDiagnoses) {
		super(diagnoser);
		this.writeDiagnoses = writeDiagnoses;
		XMLOutputFactory factory = XMLOutputFactory.newInstance();

		try {
			xtw = factory.createXMLStreamWriter(stream);
		} catch (XMLStreamException e) {
			// TODO: not very clean, this :)
			throw new RuntimeException(e);
		}
	}

	private void addDiagnoses(List<Diagnosis> diagnoses) throws XMLStreamException {
		xtw.writeStartElement("diagnoses");
		xtw.writeCharacters("\n");
		for (Diagnosis diagnosis : diagnoses) {
			addDiagnosis(diagnosis);
		}
		xtw.writeEndElement();
		xtw.writeCharacters("\n");
	}

	private void addDiagnosis(Diagnosis diagnosis) throws XMLStreamException {
		xtw.writeStartElement("diagnosis");
		xtw.writeCharacters("\n");

		for (Environment env : diagnosis.getDiagnosis()) {
			addEnvironment(env);
		}

		xtw.writeEndElement();
		xtw.writeCharacters("\n");
	}

	public void addConflict(ConflictEvent conflictEvent) throws XMLStreamException {
		xtw.writeStartElement("conflict");
		xtw.writeAttribute("serial_source", "0");
		xtw.writeAttribute("timestamp", conflictEvent.getTimeStamp().toString());
		xtw.writeAttribute("source", Integer.toString(conflictEvent.getConflict().getSource()));
		xtw.writeAttribute("id", Integer.toString(conflictEvent.getConflict().getId()));

		xtw.writeCharacters("\n");
		for (Environment env : conflictEvent.getConflict().getConflict()) {
			addEnvironment(env);
		}

		xtw.writeEndElement();
		xtw.writeCharacters("\n");
}

//	public void addTimeStamp(Date date) throws XMLStreamException {
//		DateFormat ISO8601Local = new SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ssZ");
//		String now = ISO8601Local.format(date);
//		now =  now.substring(0,now.length()-2) + ":" + now.substring(now.length()-2);
//
//		xtw.writeStartElement("receivedate");
//		xtw.writeCharacters(now);
//		xtw.writeEndElement();
//	}

	public void addEnvironment(Environment environment) throws XMLStreamException {
		xtw.writeCharacters("\t");
		
		if (environment instanceof NodeEnvironment) {
			NodeEnvironment nodeEnv = (NodeEnvironment) environment;
			xtw.writeEmptyElement("env_node");
			xtw.writeAttribute("nodeId", Integer.toString(nodeEnv.getNodeID()));
		} else if (environment instanceof SensorEnvironment) {
			SensorEnvironment sensorEnv = (SensorEnvironment) environment;
			xtw.writeEmptyElement("env_sensor");
			xtw.writeAttribute("nodeId", Integer.toString(sensorEnv.getNodeID()));
		} else if (environment instanceof LinkEnvironment) {
			LinkEnvironment linkEnv = (LinkEnvironment) environment;
			xtw.writeEmptyElement("env_link");
			xtw.writeAttribute("sender", Integer.toString(linkEnv.getSender()));
			xtw.writeAttribute("receiver", Integer.toString(linkEnv.getReceiver()));
		}
		xtw.writeCharacters("\n");
	}

	public void addConflictCancellation(ConflictCancellationEvent confCancel) throws XMLStreamException {
		xtw.writeEmptyElement("conflict_cancelled");
		xtw.writeAttribute("serial_source", "0");
		xtw.writeAttribute("timestamp", confCancel.getTimeStamp().toString());
		xtw.writeAttribute("source", Integer.toString(confCancel.getSource()));
		xtw.writeAttribute("id", Integer.toString(confCancel.getId()));

//		addTimeStamp(new Date());
//		xtw.writeEndElement();
		xtw.writeCharacters("\n");
	}

	public void addMessage(MessageEvent messageEvent) throws XMLStreamException {
		xtw.writeEmptyElement("conflict_message");
		xtw.writeAttribute("timestamp", messageEvent.getTimeStamp().toString());
		xtw.writeAttribute("serial_source", Integer.toString(messageEvent.getNodeId()));
		xtw.writeCharacters("\n");
	}

	public void addNodeDeath(NodeDeathEvent nodeEvent) throws XMLStreamException {
		xtw.writeEmptyElement("node_death_message");
		xtw.writeAttribute("timestamp", nodeEvent.getTimeStamp().toString());
		xtw.writeAttribute("serial_source", Integer.toString(nodeEvent.getNodeId()));
		xtw.writeAttribute("source", Integer.toString(nodeEvent.getNodeId()));
		xtw.writeCharacters("\n");
	}

	public void addSensorDeath(SensorDeathEvent sensorEvent) throws XMLStreamException {
		xtw.writeEmptyElement("sensor_death_message");
		xtw.writeAttribute("timestamp", sensorEvent.getTimeStamp().toString());
		xtw.writeAttribute("serial_source", Integer.toString(sensorEvent.getNodeId()));
		xtw.writeAttribute("source", Integer.toString(sensorEvent.getNodeId()));
		xtw.writeCharacters("\n");
	}	
	// Print messages to console. 
	@Override
	public void update(Observable o, Object arg) {
		if (writeDiagnoses)
			super.update(o, arg);
		try {
			if (arg instanceof ConflictEvent) {
				addConflict(((ConflictEvent) arg));
				if (writeDiagnoses)
					addDiagnoses(getDiagnoses());
			}
			else if (arg instanceof ConflictCancellationEvent) {
				ConflictCancellationEvent confCancel = (ConflictCancellationEvent) arg;
				addConflictCancellation(confCancel);
				if (writeDiagnoses)
					addDiagnoses(getDiagnoses());
			}
			else if (arg instanceof EndEvent) {
				xtw.writeEndElement();
				xtw.writeEndDocument();
			}
			else if (arg instanceof StartEvent) {
				xtw.writeStartDocument();
				xtw.writeCharacters("\n");
				xtw.writeStartElement("run");
				xtw.writeCharacters("\n");
			} else if (arg instanceof MessageEvent) {
				addMessage((MessageEvent) arg);
			} else if (arg instanceof SensorDeathEvent) {
				addSensorDeath ((SensorDeathEvent) arg);
			} else if (arg instanceof NodeDeathEvent) {
				addNodeDeath ((NodeDeathEvent) arg);
			}
			xtw.flush();
		} catch (XMLStreamException e) {
			// TODO: not very clean, this :)
			throw new RuntimeException(e);
		}
	}
}
