#ifndef MODEL_H
#define MODEL_H
#include "Observations.h"

#define RULE_TYPE "RuleType"
#define EMPTY_RULE ((RULE_TYPE_NONE << 1) | 1)

/*
 * Rule types, ensure that when adding a new rule,
 * 1) a new type is added
 * 2) The rule_info array is updated
 * 3) (optional) a struct representing the rule is created.
 */
enum {
	RULE_TYPE_NONE = unique(RULE_TYPE),
	RULE_TYPE_IMPLICATION = unique(RULE_TYPE),
	RULE_TYPE_ABOUT_EQUALS = unique(RULE_TYPE),
	RULE_TYPE_COUNT = uniqueCount(RULE_TYPE)
};

typedef struct rule_info {
	uint8_t num_environments;
	uint8_t num_observations;
} rule_info_t;

// Cannot use RULE_TYPE_COUNT here, since nesC gives an assertion error.
rule_info_t const rule_info[3] =
{
		{0, 0},
		{1, 2},
		{2, 2}
};

typedef struct rule_implication {
    observation_id_t obs_id_1;
    observation_id_t obs_id_2;
    environment_id_t env_id_1;
} rule_implication_t;

typedef struct rule_about_equals {
    observation_id_t obs_id_1;
    observation_id_t obs_id_2;
    environment_id_t env_id_1;
    environment_id_t env_id_2;
} rule_about_equals_t;

#endif
